unit urotatebitmap;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, Buttons, LCLType,
     IntfGraphics, fpImage, LCLIntf, Types;
  
type
    TRotateDirection = (rdRight, rdNormal, rdLeft);

    TRotatedBitmap = class
    private
        FNormalImage, FRotatedImage : TLazIntfImage;
        FRotateTo : TRotateDirection;
        function GetBitmap : TBitmap;
        procedure SetRotateTo(const Value : TRotateDirection);
        procedure DoRotate;
    public
        constructor Create;virtual;
        destructor Destroy;virtual;
        procedure LoadBitmap(b : TBitmap);
        property Bitmap : TBitmap read GetBitmap;
        property Image : TLazIntfImage read FRotatedImage;
        property RotateDirection : TRotateDirection read FRotateTo write SetRotateTo;
        procedure Draw(X,Y: Integer;var b : TBitmap); virtual;
        procedure Draw(X,Y: Integer;var b : TLazIntfImage); virtual;
        procedure Draw(X,Y: Integer;var b : TBitmap;
                  TransparentColor : TColor); virtual;
        procedure Draw(X,Y: Integer;var b : TLazIntfImage;
                  TransparentColor : TFPColor); virtual;
    end;

    TRotatedGlyph = class(TRotatedBitmap)
    private
        FNormalGlyphBitmap : TBitmap;
        FNormalGlyph : TButtonGlyph;
        FTransparentColor : TColor;
        FButtonState : TButtonState;
        function GetGlyph : TBitmap;
        procedure SetGlyph(Value: TBitmap);
        procedure SetButtonState(Value: TButtonState);
        procedure SetTransparentColor(Value: TColor);
    public
        constructor Create; override;
        destructor Destroy; override;
        procedure Draw(X,Y: Integer;var b : TBitmap;
                  TransparentColor : TColor); override;
        procedure Draw(X, Y: Integer; var b: TLazIntfImage;
           TransparentColor: TFPColor); override;
        procedure Update;
        property State : TButtonState read FButtonState write SetButtonState;
        property Glyph : TBitmap read GetGlyph write SetGlyph;
        property TransparentColor: TColor read FTransparentColor write SetTransparentColor;
    end;

    TRotatedText = class(TRotatedBitmap)
    private
       FBitmap : TBitmap;
       FText : String;
       procedure SetText(Value: String);
       function GetCanvas : TCanvas;
    public
       constructor Create; override;
       destructor Destroy; override;
       procedure Update;
       property Text : String read FText write SetText;
       property Canvas : TCanvas read GetCanvas;
       procedure Draw(X,Y: Integer;var b : TBitmap); override;
       procedure Draw(X,Y: Integer;var b : TLazIntfImage); override;
    end;

    operator := (const b : TBitmap) : TLazIntfImage;
    operator := (const i : TLazIntfImage) : TBitmap;

implementation

constructor TRotatedGlyph.Create;
begin
    inherited;

    FNormalGlyph := TButtonGlyph.Create;
    //FNormalGlyph.SetTransparentMode(gtmTransparent);

    FTransparentColor:=clFuchsia;
    FNormalGlyphBitmap := TBitmap.Create;
    
    FButtonState := bsUp;
end;

destructor TRotatedGlyph.Destroy;
begin
    FNormalGlyph.Free;
    FNormalGlyphBitmap.Free;

    inherited;
end;

procedure TRotatedGlyph.SetTransparentColor(Value: TColor);
begin
    FTransparentColor:=Value;
    
    Update;
end;

function TRotatedGlyph.GetGlyph : TBitmap;
begin
    Result := FNormalGlyphBitmap;
end;

procedure TRotatedGlyph.SetGlyph(Value: TBitmap);
begin
    FNormalGlyphBitmap := Value;
    //FNormalGlyph.Glyph.TransparentMode:=tmFixed;
    //FNormalGlyph.Glyph.Transparent:=true;
    
    Update;
end;

procedure TRotatedGlyph.SetButtonState(Value: TButtonState);
begin
    FButtonState:=Value;

    Update;
end;

procedure TRotatedGlyph.Update;
var
   TempBitmap : TBitmap;
   SrcIntf, TrgIntf : TLazIntfImage;
   i, j : Integer;
begin
    TempBitmap := TBitmap.Create;
    TempBitmap.Width:=FNormalGlyphBitmap.Width;
    TempBitmap.Height:=FNormalGlyphBitmap.Height;

    TempBitmap.Canvas.Brush.Color:=clNone;
    TempBitmap.Canvas.FillRect(0,0,TempBitmap.Width,TempBitmap.Height);
    
    SrcIntf := FNormalGlyphBitmap;
    
    TrgIntf := TempBitmap;
    {TmpIntf.DataDescription := GetDescriptionFromDevice(0);
    TmpIntf.SetSize(TempBitmap.Width, TempBitmap.Height);
     }
    
    for i := 0 to TempBitmap.Width-1 do
      for j := 0 to TempBitmap.Height-1 do
        if SrcIntf.Colors[i,j] <> TColorToFPColor(FTransparentColor) then
         TrgIntf.Colors[i,j] := SrcIntf.Colors[i,j];
    
    {FNormalGlyph.Draw(TempBitmap.Canvas,Rect(0,0,TempBitmap.Width,TempBitmap.Height),
      Point(0,0), FButtonState, true, 0);
    }

    TempBitmap := TrgIntf;
    
    LoadBitmap(TempBitmap);

    TempBitmap.Free;
end;

procedure TRotatedGlyph.Draw(X,Y: Integer;var b : TBitmap;
                  TransparentColor : TColor);
var
   Temp : TLazIntfImage;
begin
    Temp := b;
    Draw(X,Y,Temp, TColorToFPColor(TransparentColor));
    b := Temp;
    
    Temp.Free;
end;

procedure TRotatedGlyph.Draw(X, Y: Integer; var b: TLazIntfImage;
           TransparentColor: TFPColor);
var
   TempBitmap,Trg : TBitmap;
begin
   Trg := TBitmap.Create;

   //First Rotate the Glyph then Draw it with an other State
   inherited Draw(X,Y,b,TransparentColor);
   
   TempBitmap := b;
   
   Trg.Width:=b.Width;
   Trg.Height:=b.Height;
   
   FNormalGlyph.Glyph.Assign(TempBitmap);
   
   FNormalGlyph.Draw(Trg.Canvas,Rect(0,0,Trg.Width,Trg.Height),
      Point(0,0), FButtonState, true, 0);
      
   b := Trg;
   
   Trg.Free;
   TempBitmap.Free;
end;

constructor TRotatedText.Create;
begin
    inherited;

    FBitmap := TBitmap.Create;
    FText:='X';
    SetText('');
end;

destructor TRotatedText.Destroy;
begin
    FBitmap.Free;

    inherited;
end;

procedure TRotatedText.Update;
begin
    SetText(FText);
end;

procedure TRotatedText.SetText(Value: String);
var
   TextSize : TSize;
begin
    if FText <> Value then
    begin
       FText:=Value;

       TextSize := FBitmap.Canvas.TextExtent(FText);

       FBitmap.Width:=TextSize.cx;
       FBitmap.Height:=TextSize.cy;

       FBitmap.Canvas.FillRect(0,0,FBitmap.Width, FBitmap.Height);
       
       FBitmap.Canvas.TextOut(0,0,FText);

       Inherited LoadBitmap(FBitmap);
    end;
end;

function TRotatedText.GetCanvas : TCanvas;
begin
    Result := FBitmap.Canvas;
end;

procedure TRotatedText.Draw(X,Y: Integer;var b : TBitmap);
begin
    Inherited Draw(X,Y,b,FBitmap.Canvas.Brush.Color);
end;

procedure TRotatedText.Draw(X,Y: Integer;var b : TLazIntfImage);
begin
    Inherited Draw(X,Y,b,TColorToFPColor(FBitmap.Canvas.Brush.Color));
end;

operator := (const b : TBitmap) : TLazIntfImage;
begin
  Result := TLazIntfImage.Create(0,0);
  Result.LoadFromBitmap(b.Handle,b.MaskHandle);
end;

operator := (const i : TLazIntfImage) : TBitmap;
var
   ImageHandle, MaskHandle : HBitmap;
begin
  Result := TBitmap.Create;
  i.CreateBitmaps(ImageHandle, MaskHandle);
  Result.Handle:=ImageHandle;
  Result.MaskHandle:=MaskHandle;
end;

constructor TRotatedBitmap.Create;
begin
     //inherited;

     FRotateTo:=rdNormal;
end;

destructor TRotatedBitmap.Destroy;
begin
     if Assigned(FNormalImage) then FNormalImage.Free;
     if Assigned(FRotatedImage) then FRotatedImage.Free;
end;

function TRotatedBitmap.GetBitmap : TBitmap;
var
   TempIntf : TLazIntfImage;
begin
   if FRotateTo = rdNormal then
      TempIntf := FNormalImage
   else
      TempIntf := FRotatedImage;

    if Assigned(TempIntf) then
       Result := TempIntf;
end;

procedure TRotatedBitmap.SetRotateTo(const Value : TRotateDirection);
begin
    //WriteLn(FRotateTo<>Value);
    if FRotateTo<>Value then
    begin
        //WriteLn('SetRotateTo');
        FRotateTo:=Value;

        DoRotate;
    end;
end;

procedure TRotatedBitmap.LoadBitmap(b : TBitmap);
begin
   if Assigned(FNormalImage) then FNormalImage.Free;
   if not Assigned(FRotatedImage) then begin
       FRotatedImage := TLazIntfImage.Create(0,0);
       FRotatedImage.DataDescription := GetDescriptionFromDevice(0);
   end;

   FNormalImage := b;

   if FRotateTo <> rdNormal then DoRotate;
end;

procedure TRotatedBitmap.DoRotate;
var
   px, py, iw, ih, nx, ny : Integer;
   CurColor: TFPColor;
begin
   if FRotateTo=rdNormal then Exit;

   {if Assigned(FRotatedImage) then
      FRotatedImage.Free;

   FRotatedImage := TLazIntfImage.Create(0,0);

   FRotatedImage.DataDescription := GetDescriptionFromDevice(0);
    }
   FRotatedImage.SetSize({FNormalImage.Width,FNormalImage.Height}
      FNormalImage.Height,FNormalImage.Width);

   FRotatedImage.FillPixels(colWhite);

   //WriteLn(FRotatedImage.Width, ' ', FRotatedImage.Height);

   for px := 0 to FNormalImage.Width-1 do
    for py := 0 to FNormalImage.Height-1 do
     begin
         if FRotateTo = rdRight then
         begin
             nx := FRotatedImage.Width-1-py;
             ny := px;
         end else begin
             nx := py;
             ny := FRotatedImage.Height-1-px;
         end;

         //WriteLn(nx, ' ', ny, ' ', px, ' ', py);

         CurColor := FNormalImage.Colors[px,py];

         FRotatedImage.Colors[nx,ny]:= CurColor;
     end;
end;

procedure TRotatedBitmap.Draw(X,Y: Integer;var b : TBitmap);
begin
    Draw(X,Y,b,clNone);
end;

procedure TRotatedBitmap.Draw(X,Y: Integer;var b : TLazIntfImage);
begin
    Draw(X,Y,b,TColorToFPColor(clNone));
end;

procedure TRotatedBitmap.Draw(X,Y: Integer;var b : TBitmap; TransparentColor : TColor);
var
   TempIntfImage : TLazIntfImage;
begin
   TempIntfImage := b;
   b.Free;
   Draw(X,Y, TempIntfImage, TColorToFPColor(TransparentColor));
   b := TempIntfImage;
   TempIntfImage.Free;
end;

procedure TRotatedBitmap.Draw(X,Y: Integer;var b : TLazIntfImage; TransparentColor : TFPColor);
var
   px, py : Integer;
   TempIntf : TLazIntfImage;
begin
   if FRotateTo = rdNormal then
      TempIntf := FNormalImage
   else
      TempIntf := FRotatedImage;

   //WriteLn(Assigned(FNormalImage));
   //WriteLn(Assigned(FRotatedImage));
         
   for px := 0 to TempIntf.Width-1 do
    for py := 0 to TempIntf.Height-1 do
     if (TempIntf.Colors[px,py] <> TransparentColor)
     AND ((X+px) < b.Width) AND ((Y+py) < b.Height)
     AND ((X+px) >= 0) AND ((Y+py) >= 0) then
     begin
      //WriteLn(px, ' ', py);
      b.Colors[X+px,Y+py]:=TempIntf.Colors[px,py];
     end;
end;

end.

